$TITLE  SAMBAL
$STITLE SAM BALANCING PROGRAM

*1. Input names of matrix accounts

Set
I MATRIX ACCOUNTS
/
A_AGRC
A_MING
A_UTIL
A_CNST
A_MANU
A_TRDE
A_OSER
A_EDUS
A_HEAS
A_PADM
C_AGRC
C_MING
C_UTIL
C_CNST
C_MANU
C_TRDE
C_OSER
C_EDUS
C_HEAS
C_PADM
TC_DOME
TC_IMPT
TC_EXPT
F_FLAB
F_ILAB
F_CAPR
T_LBTX
T_PYTX
T_CPTX
T_SLTX
T_AMTX
T_INTX
T_CITX
T_SBPD
I_HHD1
I_HHD2
I_HHD3
I_HHD4
I_HHD5
I_FRMS
I_GOVN
I_GOVR
I_FOOD
I_ELDY
I_EDUC
I_CHLD
I_ENGY
I_PROC
I_UNPE
I_PALS
I_PEMT
I_TRFR
I_GRTS
I_SIAC
I_ROWD
I_TOTL
/

INT(I) All accounts except total
ALIAS(I,J);
ALIAS(INT,JNT);
INT(I) = YES;
INT("I_TOTL") = NO;

*2. Input initial matrix

$CALL "GDXXRW i=SAM_2012v5.xls o=SAM_2012v5.gdx par=SAM0 rdim=1 cdim=1 rng=SAM!A1:BD56"

PARAMETER SAM0(I,J);

$GDXIN SAM_2012v5.gdx
*$onUNDF
$LOAD SAM0
$GDXIN
Display SAM0;


*3. Position of negative values in unbalanced matrix are noted
PARAMETER
NEG(I,J) Matrix of negative values;
NEG(INT,JNT)$[(SAM0(INT,JNT) LT 0)
AND (SAM0(JNT,INT)-SAM0(INT,JNT) GT 0)
AND (SAM0(JNT,INT)+SAM0(INT,JNT) LT 0)]=1;

*Negative matrix values are transposed
SAM0(INT,JNT)$[(SAM0(JNT,INT) LT 0)
AND (SAM0(INT,JNT)-SAM0(JNT,INT) GT 0)]=SAM0(INT,JNT)-SAM0(JNT,INT);
SAM0(JNT,INT)$[(SAM0(JNT,INT) LT 0)
AND (SAM0(INT,JNT)-SAM0(JNT,INT) GT 0)]=0;

*4. Initial matrix values transformed in proportion
PARAMETER
TOTO total of initial matrix ;
TOTO = SUM((INT,JNT), SAM0(INT,JNT));
SAM0(INT,JNT) = SAM0(INT,JNT)/TOTO;

*5. Non-zero log parameter
SCALARS
delta Non-zero log parameter;
delta =.0000000000001;

*6. Definition of variables
VARIABLES
SAM(I,J) New SAM with transposed negative values
OPT Distance variable;

*7. Program equations
EQUATIONS
OPTIMIZE Optimization criterion
CONSTRAINT(I) Equality between matrix and row sums
CONSTRAINT1 Sum of proportions equals one
;

*Entropy optimization criterion
OPTIMIZE..
OPT =E= SUM((INT,JNT)$(SAM0(INT,JNT) NE 0),(SAM(INT,JNT))*(LOG(SAM(INT,JNT)+delta)-LOG(SAM0(INT,JNT)+delta)));

*Equality between row and column sums
CONSTRAINT(INT)..
SUM(JNT,SAM(INT,JNT))=E=SUM(JNT,SAM(JNT,INT));

*Proportions sum equal one
CONSTRAINT1..
SUM((INT,JNT),SAM(INT,JNT))=E=1 ;

*8. Cell values between 0 and infinity and empty cells remain empty
SAM.L(I,J) = SAM0(I,J);
SAM.LO(INT,JNT) = 0 ;
SAM.UP(INT,JNT) = +INF ;
SAM.FX(INT,JNT)$(NOT SAM0(INT,JNT)) = 0 ;
OPT.L = 0;

*9. Model
MODEL SAMBAL / ALL /;
* Model attributes (if desired)
* the "workspace" attribute tells the solver how much workspace in Megabytes
* to allocate for problem solution.
*SAMBAL.workspace = 10;
* The "optfile" attribute tells the solver to use a solver options file.
*SAMBAL.optfile = 1;
* OPTION statements
* Choose solver
* OPTION NLP = MINOS5;
* OPTION NLP = CONOPT;
* OPTION NLP = CONOPT2;
OPTION NLP = CONOPT3;

* The "iterlim" option sets a limit on the number of solver iterations
OPTION iterlim = 99999;
SOLVE SAMBAL USING NLP MINIMIZING OPT;

*10. RESULTS COPIED TO A NEW MATRIX
PARAMETER
NSAM(I,J) New (balanced) matrix;
NSAM(INT,JNT)=SAM.L(INT,JNT);

NSAM(INT,JNT)$(NEG(INT,JNT)=1)=-NSAM(JNT,INT);
NSAM(JNT,INT)$(NEG(INT,JNT)=1)=0;

* Transformation of proportions into SAM transaction flow
NSAM(INT,JNT) = NSAM(INT,JNT)*TOTO;

* Export results first to a GDX file, then to an Excel file
execute_unload "Results", NSAM;
*execute '=gdx2xls Results.gdx';
Execute 'GDXXRW.EXE Results.gdx O=NewSAM.xls par=NSAM rng=NSAM!A1:BE57 rdim=1 cdim=1';

*11. Problem detection
PARAMETER
PROBS(I,J);
PROBS(I,J)=0;
PROBS(I,J)$(NSAM(I,J) EQ 0 AND SAM0(I,J) NE 0)=SAM0(I,J);
DISPLAY PROBS;
